import { CompletingTransformer, Dictionary, QuittingReducer, Transducer } from "./types";
export interface TransformChain<T> {
    compose<U>(transducer: Transducer<T, U>): TransformChain<U>;
    dedupe(): TransformChain<T>;
    drop(n: number): TransformChain<T>;
    dropWhile(pred: (item: T) => boolean): TransformChain<T>;
    filter(pred: (item: T) => boolean): TransformChain<T>;
    flatMap<U>(f: (item: T) => Iterable<U>): TransformChain<U>;
    interpose(separator: T): TransformChain<T>;
    map<U>(f: (item: T) => U): TransformChain<U>;
    mapIndexed<U>(f: (item: T, index: number) => U): TransformChain<U>;
    partitionAll(n: number): TransformChain<T[]>;
    partitionBy(pred: (item: T) => any): TransformChain<T[]>;
    remove(pred: (item: T) => boolean): TransformChain<T>;
    removeAbsent(): TransformChain<NonNullable<T>>;
    take(n: number): TransformChain<T>;
    takeNth(n: number): TransformChain<T>;
    takeWhile(pred: (item: T) => boolean): TransformChain<T>;
    reduce<TResult>(reducer: QuittingReducer<TResult, T>, initialValue: TResult): TResult;
    reduce<TResult, TCompleteResult>(transformer: CompletingTransformer<TResult, TCompleteResult, T>): TCompleteResult;
    count(): number;
    every(pred: (item: T) => boolean): boolean;
    find(pred: (item: T) => boolean): T | null;
    first(): T | null;
    forEach(f: (item: T) => void): void;
    isEmpty(): boolean;
    joinToString(separator: string): string;
    some(pred: (item: T) => boolean): boolean;
    toArray(): T[];
    toMap<K, V>(getKey: (item: T) => K, getValue: (item: T) => V): Map<K, V>;
    toMapGroupBy<K>(getKey: (item: T) => K): Map<K, T[]>;
    toMapGroupBy<K, V>(getKey: (item: T) => K, transformer: CompletingTransformer<any, V, T>): Map<K, V>;
    toObject<U>(getKey: (item: T) => string, getValue: (item: T) => U): Dictionary<U>;
    toObjectGroupBy(getKey: (item: T) => string): Dictionary<T[]>;
    toObjectGroupBy<U>(getKey: (item: T) => string, transformer: CompletingTransformer<any, U, T>): Dictionary<U>;
    toSet(): Set<T>;
    toIterator(): IterableIterator<T>;
}
export interface TransducerBuilder<TBase, T> {
    compose<U>(transducer: Transducer<T, U>): TransducerBuilder<TBase, U>;
    dedupe(): TransducerBuilder<TBase, T>;
    drop(n: number): TransducerBuilder<TBase, T>;
    dropWhile(pred: (item: T) => boolean): TransducerBuilder<TBase, T>;
    filter(pred: (item: T) => boolean): TransducerBuilder<TBase, T>;
    flatMap<U>(f: (item: T) => Iterable<U>): TransducerBuilder<TBase, U>;
    interpose(separator: T): TransducerBuilder<TBase, T>;
    map<U>(f: (item: T) => U): TransducerBuilder<TBase, U>;
    mapIndexed<U>(f: (item: T, index: number) => U): TransducerBuilder<TBase, U>;
    partitionAll(n: number): TransducerBuilder<TBase, T[]>;
    partitionBy(pred: (item: T) => boolean): TransducerBuilder<TBase, T[]>;
    remove(pred: (item: T) => boolean): TransducerBuilder<TBase, T>;
    removeAbsent(): TransducerBuilder<TBase, NonNullable<T>>;
    take(n: number): TransducerBuilder<TBase, T>;
    takeNth(n: number): TransducerBuilder<TBase, T>;
    takeWhile(pred: (item: T) => boolean): TransducerBuilder<TBase, T>;
    build(): Transducer<TBase, T>;
}
export declare function chainFrom<T>(collection: Iterable<T>): TransformChain<T>;
export declare function transducerBuilder<T>(): TransducerBuilder<T, T>;
