"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("./core");
var iterables_1 = require("./iterables");
var reducers_1 = require("./reducers");
var transducers_1 = require("./transducers");
function chainFrom(collection) {
    return new TransducerChain(collection);
}
exports.chainFrom = chainFrom;
function transducerBuilder() {
    return new TransducerChain([]);
}
exports.transducerBuilder = transducerBuilder;
var TransducerChain = /** @class */ (function () {
    function TransducerChain(collection) {
        this.collection = collection;
        this.transducers = [];
    }
    TransducerChain.prototype.compose = function (transducer) {
        this.transducers.push(transducer);
        return this;
    };
    TransducerChain.prototype.build = function () {
        var _this = this;
        return function (x) {
            var result = x;
            for (var i = _this.transducers.length - 1; i >= 0; i--) {
                result = _this.transducers[i](result);
            }
            return result;
        };
    };
    // ----- Composing transducers -----
    TransducerChain.prototype.dedupe = function () {
        return this.compose(transducers_1.dedupe());
    };
    TransducerChain.prototype.drop = function (n) {
        return this.compose(transducers_1.drop(n));
    };
    TransducerChain.prototype.dropWhile = function (pred) {
        return this.compose(transducers_1.dropWhile(pred));
    };
    TransducerChain.prototype.filter = function (pred) {
        return this.compose(transducers_1.filter(pred));
    };
    TransducerChain.prototype.flatMap = function (f) {
        return this.compose(transducers_1.flatMap(f));
    };
    TransducerChain.prototype.interpose = function (separator) {
        return this.compose(transducers_1.interpose(separator));
    };
    TransducerChain.prototype.map = function (f) {
        return this.compose(transducers_1.map(f));
    };
    TransducerChain.prototype.mapIndexed = function (f) {
        return this.compose(transducers_1.mapIndexed(f));
    };
    TransducerChain.prototype.partitionAll = function (n) {
        return this.compose(transducers_1.partitionAll(n));
    };
    TransducerChain.prototype.partitionBy = function (f) {
        return this.compose(transducers_1.partitionBy(f));
    };
    TransducerChain.prototype.remove = function (pred) {
        return this.compose(transducers_1.remove(pred));
    };
    TransducerChain.prototype.removeAbsent = function () {
        return this.remove(function (item) { return item == null; });
    };
    TransducerChain.prototype.take = function (n) {
        return this.compose(transducers_1.take(n));
    };
    TransducerChain.prototype.takeNth = function (n) {
        return this.compose(transducers_1.takeNth(n));
    };
    TransducerChain.prototype.takeWhile = function (pred) {
        return this.compose(transducers_1.takeWhile(pred));
    };
    TransducerChain.prototype.reduce = function (reducer, initialValue) {
        if (typeof reducer === "function") {
            // Type coercion because in this branch, TResult and TCompleteResult are
            // the same, but the checker doesn't know that.
            return core_1.transduce(this.collection, this.build(), reducer, initialValue);
        }
        else {
            return core_1.transduce(this.collection, this.build(), reducer);
        }
    };
    TransducerChain.prototype.count = function () {
        return this.reduce(reducers_1.count());
    };
    TransducerChain.prototype.every = function (pred) {
        return this.reduce(reducers_1.every(pred));
    };
    TransducerChain.prototype.find = function (pred) {
        return this.reduce(reducers_1.find(pred));
    };
    TransducerChain.prototype.first = function () {
        return this.reduce(reducers_1.first());
    };
    TransducerChain.prototype.forEach = function (f) {
        this.reduce(reducers_1.forEach(f));
    };
    TransducerChain.prototype.isEmpty = function () {
        return this.reduce(reducers_1.isEmpty());
    };
    TransducerChain.prototype.joinToString = function (separator) {
        return this.reduce(reducers_1.joinToString(separator));
    };
    TransducerChain.prototype.some = function (pred) {
        return this.reduce(reducers_1.some(pred));
    };
    TransducerChain.prototype.toArray = function () {
        return this.reduce(reducers_1.toArray());
    };
    TransducerChain.prototype.toMap = function (getKey, getValue) {
        return this.reduce(reducers_1.toMap(getKey, getValue));
    };
    TransducerChain.prototype.toMapGroupBy = function (getKey, transformer) {
        return this.reduce(reducers_1.toMapGroupBy(getKey, transformer));
    };
    TransducerChain.prototype.toObject = function (getKey, getValue) {
        return this.reduce(reducers_1.toObject(getKey, getValue));
    };
    TransducerChain.prototype.toObjectGroupBy = function (getKey, transformer) {
        return this.reduce(reducers_1.toObjectGroupBy(getKey, transformer));
    };
    TransducerChain.prototype.toSet = function () {
        return this.reduce(reducers_1.toSet());
    };
    TransducerChain.prototype.toIterator = function () {
        return iterables_1.lazyTransduce(this.collection, this.build());
    };
    return TransducerChain;
}());
//# sourceMappingURL=chain.js.map