"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var iterables_1 = require("./iterables");
var propertyNames_1 = require("./propertyNames");
var util_1 = require("./util");
function transduce(collection, transform, reducer, initialValue) {
    var transformer;
    if (typeof reducer === "function") {
        // Type coercion because in this branch, TResult and TCompleteResult are
        // the same, but the checker doesn't know that.
        transformer = new ReducerWrappingTransformer(reducer, initialValue);
    }
    else {
        transformer = reducer;
    }
    return reduceWithTransformer(collection, transform(transformer));
}
exports.transduce = transduce;
function reduceWithTransformer(collection, f) {
    var uncompleteResult = reduceWithFunction(collection, f[propertyNames_1.STEP].bind(f), f[propertyNames_1.INIT]());
    return f[propertyNames_1.RESULT](util_1.unreduced(uncompleteResult));
}
function reduceWithFunction(collection, f, initialValue) {
    var iterator = iterables_1.getIterator(collection);
    var result = initialValue;
    while (true) {
        var input = iterator.next();
        if (input.done) {
            return result;
        }
        var next = f(result, input.value);
        if (util_1.isReduced(next)) {
            return next;
        }
        else {
            result = next;
        }
    }
}
exports.reduceWithFunction = reduceWithFunction;
var ReducerWrappingTransformer = /** @class */ (function () {
    function ReducerWrappingTransformer(f, initialValue) {
        this.initialValue = initialValue;
        this[propertyNames_1.STEP] = f;
    }
    ReducerWrappingTransformer.prototype[propertyNames_1.INIT] = function () {
        return this.initialValue;
    };
    ReducerWrappingTransformer.prototype[propertyNames_1.RESULT] = function (result) {
        return result;
    };
    return ReducerWrappingTransformer;
}());
//# sourceMappingURL=core.js.map