"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("./core");
var propertyNames_1 = require("./propertyNames");
var util_1 = require("./util");
function updateValue(result, newValue) {
    if (util_1.isReduced(newValue)) {
        result.value = newValue[propertyNames_1.VALUE];
        return util_1.reduced(result);
    }
    else {
        result.value = newValue;
        return result;
    }
}
var Dedupe = /** @class */ (function () {
    function Dedupe(xf) {
        this.xf = xf;
    }
    Dedupe.prototype[propertyNames_1.INIT] = function () {
        return { last: {}, value: this.xf[propertyNames_1.INIT]() };
    };
    Dedupe.prototype[propertyNames_1.RESULT] = function (result) {
        return this.xf[propertyNames_1.RESULT](result.value);
    };
    Dedupe.prototype[propertyNames_1.STEP] = function (result, input) {
        if (input !== result.last) {
            result.last = input;
            return updateValue(result, this.xf[propertyNames_1.STEP](result.value, input));
        }
        else {
            return result;
        }
    };
    return Dedupe;
}());
function dedupe() {
    return function (xf) { return new Dedupe(xf); };
}
exports.dedupe = dedupe;
var Drop = /** @class */ (function () {
    function Drop(xf, n) {
        this.xf = xf;
        this.n = n;
        this.i = 0;
    }
    Drop.prototype[propertyNames_1.INIT] = function () {
        return this.xf[propertyNames_1.INIT]();
    };
    Drop.prototype[propertyNames_1.RESULT] = function (result) {
        return this.xf[propertyNames_1.RESULT](result);
    };
    Drop.prototype[propertyNames_1.STEP] = function (result, input) {
        return this.i++ < this.n ? result : this.xf[propertyNames_1.STEP](result, input);
    };
    return Drop;
}());
function drop(n) {
    return function (xf) { return new Drop(xf, n); };
}
exports.drop = drop;
var DropWhile = /** @class */ (function () {
    function DropWhile(xf, pred) {
        this.xf = xf;
        this.pred = pred;
    }
    DropWhile.prototype[propertyNames_1.INIT] = function () {
        return { value: this.xf[propertyNames_1.INIT](), isDoneDropping: false };
    };
    DropWhile.prototype[propertyNames_1.RESULT] = function (result) {
        return this.xf[propertyNames_1.RESULT](result.value);
    };
    DropWhile.prototype[propertyNames_1.STEP] = function (result, input) {
        if (result.isDoneDropping) {
            return updateValue(result, this.xf[propertyNames_1.STEP](result.value, input));
        }
        else {
            if (this.pred(input)) {
                return result;
            }
            else {
                result.isDoneDropping = true;
                return updateValue(result, this.xf[propertyNames_1.STEP](result.value, input));
            }
        }
    };
    return DropWhile;
}());
function dropWhile(pred) {
    return function (xf) { return new DropWhile(xf, pred); };
}
exports.dropWhile = dropWhile;
var Filter = /** @class */ (function () {
    function Filter(xf, pred) {
        this.xf = xf;
        this.pred = pred;
    }
    Filter.prototype[propertyNames_1.INIT] = function () {
        return this.xf[propertyNames_1.INIT]();
    };
    Filter.prototype[propertyNames_1.RESULT] = function (result) {
        return this.xf[propertyNames_1.RESULT](result);
    };
    Filter.prototype[propertyNames_1.STEP] = function (result, input) {
        return this.pred(input) ? this.xf[propertyNames_1.STEP](result, input) : result;
    };
    return Filter;
}());
function filter(pred) {
    return function (xf) { return new Filter(xf, pred); };
}
exports.filter = filter;
var FlatMap = /** @class */ (function () {
    function FlatMap(xf, f) {
        this.xf = xf;
        this.f = f;
        this.step = xf[propertyNames_1.STEP].bind(xf);
    }
    FlatMap.prototype[propertyNames_1.INIT] = function () {
        return this.xf[propertyNames_1.INIT]();
    };
    FlatMap.prototype[propertyNames_1.RESULT] = function (result) {
        return this.xf[propertyNames_1.RESULT](result);
    };
    FlatMap.prototype[propertyNames_1.STEP] = function (result, input) {
        return core_1.reduceWithFunction(this.f(input), this.step, result);
    };
    return FlatMap;
}());
function flatMap(f) {
    return function (xf) { return new FlatMap(xf, f); };
}
exports.flatMap = flatMap;
var Interpose = /** @class */ (function () {
    function Interpose(xf, separator) {
        this.xf = xf;
        this.separator = separator;
        this.isStarted = false;
    }
    Interpose.prototype[propertyNames_1.INIT] = function () {
        return this.xf[propertyNames_1.INIT]();
    };
    Interpose.prototype[propertyNames_1.RESULT] = function (result) {
        return this.xf[propertyNames_1.RESULT](result);
    };
    Interpose.prototype[propertyNames_1.STEP] = function (result, input) {
        if (this.isStarted) {
            var withSeparator = this.xf[propertyNames_1.STEP](result, this.separator);
            if (util_1.isReduced(withSeparator)) {
                return withSeparator;
            }
            else {
                return this.xf[propertyNames_1.STEP](withSeparator, input);
            }
        }
        else {
            this.isStarted = true;
            return this.xf[propertyNames_1.STEP](result, input);
        }
    };
    return Interpose;
}());
function interpose(separator) {
    return function (xf) { return new Interpose(xf, separator); };
}
exports.interpose = interpose;
// Not named Map to avoid confusion with the native Map class.
var MapTransformer = /** @class */ (function () {
    function MapTransformer(xf, f) {
        this.xf = xf;
        this.f = f;
    }
    MapTransformer.prototype[propertyNames_1.INIT] = function () {
        return this.xf[propertyNames_1.INIT]();
    };
    MapTransformer.prototype[propertyNames_1.RESULT] = function (result) {
        return this.xf[propertyNames_1.RESULT](result);
    };
    MapTransformer.prototype[propertyNames_1.STEP] = function (result, input) {
        return this.xf[propertyNames_1.STEP](result, this.f(input));
    };
    return MapTransformer;
}());
function map(f) {
    return function (xf) { return new MapTransformer(xf, f); };
}
exports.map = map;
var MapIndexed = /** @class */ (function () {
    function MapIndexed(xf, f) {
        this.xf = xf;
        this.f = f;
    }
    MapIndexed.prototype[propertyNames_1.INIT] = function () {
        return { value: this.xf[propertyNames_1.INIT](), i: 0 };
    };
    MapIndexed.prototype[propertyNames_1.RESULT] = function (result) {
        return this.xf[propertyNames_1.RESULT](result.value);
    };
    MapIndexed.prototype[propertyNames_1.STEP] = function (result, input) {
        return updateValue(result, this.xf[propertyNames_1.STEP](result.value, this.f(input, result.i++)));
    };
    return MapIndexed;
}());
function mapIndexed(f) {
    return function (xf) { return new MapIndexed(xf, f); };
}
exports.mapIndexed = mapIndexed;
var PartitionAll = /** @class */ (function () {
    function PartitionAll(xf, n) {
        this.xf = xf;
        this.n = n;
        this.buffer = [];
    }
    PartitionAll.prototype[propertyNames_1.INIT] = function () {
        return this.xf[propertyNames_1.INIT]();
    };
    PartitionAll.prototype[propertyNames_1.RESULT] = function (result) {
        if (this.buffer.length > 0) {
            result = util_1.unreduced(this.xf[propertyNames_1.STEP](result, this.buffer));
            this.buffer = [];
        }
        return this.xf[propertyNames_1.RESULT](result);
    };
    PartitionAll.prototype[propertyNames_1.STEP] = function (result, input) {
        this.buffer.push(input);
        if (this.buffer.length === this.n) {
            var newResult = this.xf[propertyNames_1.STEP](result, this.buffer);
            this.buffer = [];
            return newResult;
        }
        else {
            return result;
        }
    };
    return PartitionAll;
}());
function partitionAll(n) {
    if (n === 0) {
        throw new Error("Size in partitionAll() cannot be 0");
    }
    else if (n < 0) {
        throw new Error("Size in partitionAll() cannot be negative");
    }
    return function (xf) { return new PartitionAll(xf, n); };
}
exports.partitionAll = partitionAll;
var INITIAL_LAST_KEY = {};
var PartitionBy = /** @class */ (function () {
    function PartitionBy(xf, f) {
        this.xf = xf;
        this.f = f;
    }
    PartitionBy.prototype[propertyNames_1.INIT] = function () {
        return {
            value: this.xf[propertyNames_1.INIT](),
            buffer: [],
            lastKey: INITIAL_LAST_KEY,
        };
    };
    PartitionBy.prototype[propertyNames_1.RESULT] = function (result) {
        if (result.buffer.length > 0) {
            result.value = util_1.unreduced(this.xf[propertyNames_1.STEP](result.value, result.buffer));
            result.buffer = [];
        }
        return this.xf[propertyNames_1.RESULT](result.value);
    };
    PartitionBy.prototype[propertyNames_1.STEP] = function (result, input) {
        var key = this.f(input);
        var value = result.value, buffer = result.buffer, lastKey = result.lastKey;
        result.lastKey = key;
        var newResult;
        if (lastKey === INITIAL_LAST_KEY || lastKey === key) {
            newResult = result;
        }
        else {
            newResult = updateValue(result, this.xf[propertyNames_1.STEP](value, buffer));
            util_1.unreduced(newResult).buffer = [];
        }
        util_1.unreduced(newResult).buffer.push(input);
        return newResult;
    };
    return PartitionBy;
}());
function partitionBy(f) {
    return function (xf) { return new PartitionBy(xf, f); };
}
exports.partitionBy = partitionBy;
var Take = /** @class */ (function () {
    function Take(xf, n) {
        this.xf = xf;
        this.n = n;
        this.i = 0;
    }
    Take.prototype[propertyNames_1.INIT] = function () {
        return this.xf[propertyNames_1.INIT]();
    };
    Take.prototype[propertyNames_1.RESULT] = function (result) {
        return this.xf[propertyNames_1.RESULT](result);
    };
    Take.prototype[propertyNames_1.STEP] = function (result, input) {
        // Written this way to avoid pulling one more element than necessary.
        if (this.n <= 0) {
            return util_1.reduced(result);
        }
        var next = this.xf[propertyNames_1.STEP](result, input);
        return this.i++ < this.n - 1 ? next : util_1.ensureReduced(next);
    };
    return Take;
}());
function remove(pred) {
    return filter(function (item) { return !pred(item); });
}
exports.remove = remove;
function take(n) {
    return function (xf) { return new Take(xf, n); };
}
exports.take = take;
var TakeNth = /** @class */ (function () {
    function TakeNth(xf, n) {
        this.xf = xf;
        this.n = n;
    }
    TakeNth.prototype[propertyNames_1.INIT] = function () {
        return { value: this.xf[propertyNames_1.INIT](), i: 0 };
    };
    TakeNth.prototype[propertyNames_1.RESULT] = function (result) {
        return this.xf[propertyNames_1.RESULT](result.value);
    };
    TakeNth.prototype[propertyNames_1.STEP] = function (result, input) {
        return result.i++ % this.n === 0
            ? updateValue(result, this.xf[propertyNames_1.STEP](result.value, input))
            : result;
    };
    return TakeNth;
}());
function takeNth(n) {
    if (n === 0) {
        throw new Error("Step in takeNth() cannot be 0");
    }
    else if (n < 0) {
        throw new Error("Step in takeNth() cannot be negative");
    }
    return function (xf) { return new TakeNth(xf, n); };
}
exports.takeNth = takeNth;
var TakeWhile = /** @class */ (function () {
    function TakeWhile(xf, pred) {
        this.xf = xf;
        this.pred = pred;
    }
    TakeWhile.prototype[propertyNames_1.INIT] = function () {
        return this.xf[propertyNames_1.INIT]();
    };
    TakeWhile.prototype[propertyNames_1.RESULT] = function (result) {
        return this.xf[propertyNames_1.RESULT](result);
    };
    TakeWhile.prototype[propertyNames_1.STEP] = function (result, input) {
        return this.pred(input)
            ? this.xf[propertyNames_1.STEP](result, input)
            : util_1.reduced(result);
    };
    return TakeWhile;
}());
function takeWhile(pred) {
    return function (xf) { return new TakeWhile(xf, pred); };
}
exports.takeWhile = takeWhile;
//# sourceMappingURL=transducers.js.map